/*
 * Purpose: wxWindows email implementation
 * Author:  Frank Bu
 * Created: 2002
 */

#ifndef EMAIL_H
#define EMAIL_H

#include <wx/wx.h>
#include <wx/filename.h>
#include <wx/wfstream.h>

#include "mime.h"

// forward declaration
class wxRecipientsIterator;

/**
 * An email without transport mechanism specific declarations or implementations.
 */
class wxEmailMessage
{
public:
	/**
	 * Constructs a new email.
	 * \param subject Subject.
	 * \param text The body text.
	 * \param from From.
	 */
	wxEmailMessage(const wxString& subject, const wxString& text, const wxString& from);

	/**
	 * Gets the from string, which was specified in the constructor.
	 * \return The from string.
	 */
	const wxString& GetFrom();

	/**
	 * Adds an additional recipient. You can add as many recipients as you want
	 * and don't have to specify it in the email message, for example if you
	 * want to send a mailing list. The RFC 821 (SMTP) garantuees 100 recipients 
	 * at once, but most 
	 * \param address Email address of the additional recipient.
	 */
	void AddRecipient(const wxString& address);

	/**
	 * Adds an additional recipient in the to-field.
	 * \param address The email address of the recipient.
	 */
	void AddTo(const wxString& address);

	/**
	 * Adds an additional recipient in the cc-field.
	 * \param address The email address of the recipient.
	 */
	void AddCc(const wxString& address);

	/**
	 * Adds an additional recipient in the bcc-field.
	 * \param address The email address of the recipient.
	 */
	void AddBcc(const wxString& address);

	// loads the file in memory and adds it to the message.

	/**
	 * Adds an file attachment. If no mime-type is specified the type will be
	 * determined by the file extension.
	 * \param fileName Filename of the new file attachment.
	 * \param mimeMainType MIME main type, like 'image'.
	 * \param mimeSubType MIME sub type, like 'jpeg'.
	 */
	void AddFile(const wxFileName& fileName, const wxString mimeMainType = "", const wxString mimeSubType = "");

	/**
	 * Returns an iterator for iterating through all recipients: to, cc and bcc.
	 * \return A new iterator for this object.
	 * \sa wxRecipientsIterator.
	 */
	wxRecipientsIterator GetRecipientsIterator();

	/**
	 * Writes the message as one blob, MIME encoded, if necessary, e.g. for sending it with SMTP.
	 * \param out The output stream to which the email has to be written.
	 */
	// 
	void Encode(wxOutputStream& out);

private:
	wxString m_subject;
	wxString m_text;
	wxString m_from;
	wxArrayString m_rcptArray;
	wxArrayString m_toArray;
	wxArrayString m_ccArray;
	wxArrayString m_bccArray;
	wxArrayMimePart m_mimeParts;

    friend class wxRecipientsIterator;
};


/**
 * Public helper class for iterating through all recipients of an email.
 */
class wxRecipientsIterator
{ 
public:
	/**
	 * Creates a new iterator for the specified wxEmailMessage object.
	 * \param emailMessage The wxEmailMessage object for which the iterator should
	 *                     iterate all recipients.
	 */
	wxRecipientsIterator(const wxEmailMessage& emailMessage);

	/**
	 * Test if there are any more email recipients to be iterated. The test doesn't increment
	 * the internal iterator counter.
	 * \return true, if there are more email recipients.
	 */
	bool HasNext();

	/**
	 * Gets the next email recipient and increments the iterator pointer.
	 * \return The next email recipient.
	 */
	wxString GetNext();

private:
	const wxEmailMessage& m_emailMessage;
	size_t m_rcptCount;
	size_t m_toCount;
	size_t m_ccCount;
	size_t m_bccCount;

};


#endif
